/*
 *  Program to print the order quantity for those inventory items
 *  whose current stocks are below their reorder levels.
 */

#include <math.h>
#include <stdio.h>

void print_heading(void);
void print_title(void);
void print_column_names(void);
void print_body(void);
int  read_data(void);
void print_report_line(void);

int    item_code, lead_time;
float  order_cost, unit_cost, interest_rate, weekly_use,
       current_stock;

/* Main Task 1 */
int main(void)
  {
    print_heading();           /* calling 1.1 */
    print_body();              /* calling 1.2 */
    return 0;
  }

/* Task 1.1 */
void print_heading(void)
  {
    print_title();              /* calling 1.1.1 */
    print_column_names();       /* calling 1.1.2 */
  }

/* Task 1.1.1 */
void print_title(void)
  {
    printf("        *****  INVENTORY REPORT  *****\n\n");
  }

/* Task 1.1.2 */
void print_column_names(void)
  {
    printf(
     " ITEM       CURRENT      REORDER        ORDER\n");
    printf(
     " CODE        STOCK        LEVEL       QUANTITY\n");
    printf(
     "______      _______      _______      ________");
    printf("\n\n");
  }

/* Task 1.2 */
void print_body(void)
  {
    /* repetitive structure: for each stock item, do 1.2.1 and 1.2.2 */
    while (read_data())        /* calling 1.2.1 */
        print_report_line();   /* calling 1.2.2 */
  }

/* Task 1.2.1 */
int read_data(void)
  {
    return scanf("%6d %6f %7f %3f %6f %2d %7f",
        &item_code, &order_cost, &unit_cost,
        &interest_rate, &weekly_use,
        &lead_time, &current_stock) != EOF ? 1 : 0;
  }

/* Task 1.2.2 */
void print_report_line(void)
  {
    float reorder_level, order_qty;

    /* Task 1.2.2.1: compute the reorder level */
    reorder_level = lead_time * weekly_use;

    /* selective structure: only if the current stock for the item
       is less than the reorder level, do 1.2.2.2 and 1.2.2.3 */
    if (current_stock < reorder_level)
      {
        /* Task 1.2.2.2: compute the economic order quantity */
        order_qty =
            sqrt((2 * 52 * weekly_use * order_cost)
                 / (unit_cost * interest_rate));
        /* Task 1.2.2.2: print data line */
        printf("%6d %12.2f %12.2f %12.2f\n", item_code,
            current_stock, reorder_level, order_qty);
      }
  }
