#include <stdio.h>
#define  MAXLINE  256

void mkidx(FILE *inpf, FILE *idxf);
void mkout(FILE *inpf, FILE *outf, FILE *idxf);

int main(int argc, char *argv[])
  {
    FILE *inpf, *outf, *idxf;

    if (argc != 3)
      {
        fprintf(stderr,
          "Usage: %s <infile> <outfile>\n", argv[0]);
        return 1;
      }

    inpf = fopen(argv[1], "r"); /* input file */
    outf = fopen(argv[2], "w"); /* output file */
    idxf = tmpfile();           /* index file */
    
    mkidx(inpf, idxf);          /* make indices */
    mkout(inpf, outf, idxf);    /* create output file */

    fclose(inpf); fclose(outf);
    return 0;
  }

void mkidx(FILE *inpf, FILE *idxf)
  {
    long lines = 0, pos = 0;
    int c;
 
    /* leave slot for the total number of lines */
    fseek(idxf, (long)sizeof(long), SEEK_SET);
    /* write index of the first line */
    fwrite((char *)&pos, sizeof(long), 1, idxf);

    for (++pos; (c = getc(inpf)) != EOF; ++pos)
        if (c == '\n')
          {
            lines++;
            /* write index of the next line */
            fwrite((char *)&pos,sizeof(long),1,idxf);
          }

    /* fill in total lines in the slot left empty at the beginning
       of the file */
    rewind(idxf);
    fwrite((char *)&lines, sizeof(long), 1, idxf);
  }

void mkout(FILE *inpf, FILE *outf, FILE *idxf)
  {
    long totlines, lineno, spos, epos;
    char buf[MAXLINE];

    rewind(idxf);
    /* determine the total number of lines */
    fread((char *)&totlines, sizeof(long), 1, idxf);

    while (scanf("%ld", &lineno) != EOF)
      {
        if (lineno <= 0 || lineno > totlines)
            continue;

        /* seek to the index for this line */
        fseek(idxf, lineno*sizeof(long), SEEK_SET);
        /* determine start of this line */
        fread((char *)&spos, sizeof(long), 1, idxf);
        /* determine start of the next line */
        fread((char *)&epos, sizeof(long), 1, idxf);

        /* seek to the start of the line in the input file */
        fseek(inpf, spos, SEEK_SET);
        /* read the line */
        fread((char *)buf, 1, epos-spos, inpf);
        /* write it out */
        fwrite((char *)buf, 1, epos-spos, outf);
      }
  }
