#include <stdio.h>
#define LARGEST     4097 /* largest element of the set */
#define WORDSIZE    32   /* assume 32 bit integer */
#define SIZEINWORDS (LARGEST/WORDSIZE+1)/* bit array size */
#define MAXONLINE   10   /* maximum elements on a print line */
#define inrange(e)  (e >= 0 && e <= LARGEST)
#define wordpos(e)  (e / WORDSIZE)
#define bitpos(e)   (e % WORDSIZE)
#define bitval(s,e) ((s[wordpos(e)]>>bitpos(e)) & 01)

typedef unsigned int SET[SIZEINWORDS];
typedef int ELEMENT;
typedef enum {FAILED, SUCCEEDED} STATUS;
typedef enum {FALSE, TRUE} BOOL;

void set_init(SET s)  /* initialize s */
  {
     register int i;

     for (i = 0; i < SIZEINWORDS; i++)  s[i] = 0;
  }

STATUS set_add(SET s, ELEMENT e)  /* add e to s */
  {
    if (!inrange(e)) return FAILED;
    s[wordpos(e)] |= 01 << bitpos(e);
    return SUCCEEDED;
  }

BOOL set_mem(SET s, ELEMENT e)  /* is e in s? */
  {
    return inrange(e) && bitval(s,e) ? TRUE : FALSE;
  }

void set_union(SET s, SET t, SET r)  /* r = s @union@ t */
  {
     register int i;

     for (i = 0; i < SIZEINWORDS; i++)
         r[i] = s[i] | t[i];
  }

void set_print(SET s) /* print members of s */
  {
     register int i, j = 0;

     printf("{");
     for (i = 0; i <= LARGEST; i++)
        if (set_mem(s,i) == TRUE)
          {
            printf("%s%d", j % MAXONLINE ? ", " :
                           j != 0 ? ",\n" : "", i);
            j++;
          }
     printf("}");
  }

void test1(void)
  {
    SET r, s, t;
    set_init(s);
    set_init(t);
    set_add(s,0);
    set_add(s,31);
    set_add(t,32);
    set_add(t,4097);
    set_union(s,t,r);

    if (set_mem(r,0)) printf("Y "); else printf("N ");
    if (set_mem(r,31)) printf("Y "); else printf("N ");
    if (set_mem(r,32)) printf("Y "); else printf("N ");
    if (set_mem(r,4097)) printf("Y "); else printf("N ");
    if (set_mem(r,2047)) printf("Y "); else printf("N ");
    if (set_mem(r,2048)) printf("Y "); else printf("N ");
    printf("\n");
    set_print(r);
    printf("\n");
}

void test2(void)
  {
    SET s;
    set_init(s);
    if (set_add(s,-1)) printf("Y "); else printf("N ");
    if (set_add(s,4098)) printf("Y "); else printf("N ");
    if (set_add(s,0)) printf("Y "); else printf("N ");
    if (set_add(s,4097)) printf("Y "); else printf("N ");
    if (set_add(s,2047)) printf("Y "); else printf("N ");
    if (set_add(s,99)) printf("Y "); else printf("N ");
    printf("\n");
    if (set_mem(s,-1)) printf("Y "); else printf("N ");
    if (set_mem(s,4098)) printf("Y "); else printf("N ");
    if (set_mem(s,0)) printf("Y "); else printf("N ");
    if (set_mem(s,4097)) printf("Y "); else printf("N ");
    if (set_mem(s,2047)) printf("Y "); else printf("N ");
    if (set_mem(s,99)) printf("Y "); else printf("N ");
    if (set_mem(s,31)) printf("Y "); else printf("N ");
    if (set_mem(s,199)) printf("Y "); else printf("N ");
    printf("\n");
    set_init(s);
    if (set_add(s,-1)) printf("Y "); else printf("N ");
    if (set_add(s,4098)) printf("Y "); else printf("N ");
    if (set_add(s,0)) printf("Y "); else printf("N ");
    if (set_add(s,4097)) printf("Y "); else printf("N ");
    if (set_add(s,2047)) printf("Y "); else printf("N ");
    if (set_add(s,99)) printf("Y "); else printf("N ");
    printf("\n");
    if (set_mem(s,-1)) printf("Y "); else printf("N ");
    if (set_mem(s,4098)) printf("Y "); else printf("N ");
    if (set_mem(s,0)) printf("Y "); else printf("N ");
    if (set_mem(s,4097)) printf("Y "); else printf("N ");
    if (set_mem(s,2047)) printf("Y "); else printf("N ");
    if (set_mem(s,99)) printf("Y "); else printf("N ");
    if (set_mem(s,31)) printf("Y "); else printf("N ");
    if (set_mem(s,199)) printf("Y "); else printf("N ");
    printf("\n");
    set_print(s);
    printf("\n");
}

int main(void)
  {
    test1();
    test2();
    return 0;
  }
