#include <stdio.h>
#define N       5           /* assume 32-bit integers */
#define INTSIZE (1 << N)    /* 32 */
#define MAXNODE 250         /* highest node number */
#define MAXROW  MAXNODE + 1 /* rows in the adjacency matrix */
#define MAXCOL  (MAXNODE >> N) + 1 /* columns */

/*
 *  returns true if the bit at position p in the integer i is 1
 */
int bitset(unsigned int num, int pos)
  {
    return num >> pos & 1;
  }

void warshall(unsigned int matrix[MAXROW][MAXCOL],
              int nodes)
  {
    int i, j, k;

    for (k = 0; k <= nodes; k++)
        for (i = 0; i <= nodes; i++)
            if (bitset(matrix[i][k >> N], k % INTSIZE))
                for (j = 0; j <= nodes >> N; j++)
                    matrix[i][j] |= matrix[k][j];
  }

int input(unsigned int matrix[MAXROW][MAXCOL])
  {
    int tail, head, maxnode = -1, greater;

    while (scanf("%d %d", &tail, &head) != EOF)
      {
        if (tail >= MAXROW || head >= MAXROW) continue;
        matrix[tail][head >> N] |= 1 << head % INTSIZE;
        greater = tail > head ? tail : head;
        if (greater > maxnode) maxnode = greater;
      }
    return maxnode;  /* the highest node number in input */
  }

void output(unsigned int matrix[MAXROW][MAXCOL],
            int nodes)
  {
    int i, j;

    for (i = 0; i <= nodes; i++)
        for (j = 0; j <= nodes; j++)
            if (bitset(matrix[i][j >> N], j % INTSIZE))
                printf("% %d\n", i, j);
  }

int main(void)
  {
    unsigned int matrix[MAXROW][MAXCOL];
    int nodes;

    nodes = input(matrix);
    warshall(matrix, nodes);
    output(matrix, nodes);
    return 0;
  }
